﻿#region File and License Information
/*
<File>
	<Copyright>Copyright © 2007, Daniel Vaughan. All rights reserved.</Copyright>
	<License see="prj:///Documentation/License.txt"/>
	<Owner Name="Daniel Vaughan" Email="dbvaughan@gmail.com"/>
	<CreationDate>2009-04-15 21:51:24Z</CreationDate>
	<LastSubmissionDate>$Date: $</LastSubmissionDate>
	<Version>$Revision: $</Version>
</File>
*/
#endregion

using System;

namespace DanielVaughan.Data
{
	/// <summary>
	/// Is raised when, during the process of deserialization, 
	/// a serialized memento is found with a higher version than 
	/// can be handled. This usually indicates an older application
	/// version is being used to open a file that was created 
	/// in a newer version.
	/// </summary>
	[Serializable]
	public class MementoIncompatibilityException : UserMessageException
	{
		/// <summary>
		/// Gets or sets the expected memento version that is supported by the memento type.
		/// </summary>
		/// <value>The expected memento version that is supported by the memento type.</value>
		public double ExpectedMementoVersion { get; private set; }

		/// <summary>
		/// Gets or sets the actual memento version that was found on the saved memento.
		/// </summary>
		/// <value>The actual memento version that was found on the saved memento.</value>
		public double ActualMementoVersion { get; private set; }

		/// <summary>
		/// Initializes a new instance of the <see cref="MementoIncompatibilityException"/> class.
		/// </summary>
		/// <param name="expectedMementoVersion">The memento version that is supported.</param>
		/// <param name="actualMementoVersion">The memento version that was found on the saved memento.</param>
		/// <param name="userMessage">The developer message that may contain 
		/// detailed information regarding the exception.</param>
		public MementoIncompatibilityException(double expectedMementoVersion, double actualMementoVersion,
			string userMessage)
			: base(userMessage)
		{
			ExpectedMementoVersion = expectedMementoVersion;
			ActualMementoVersion = actualMementoVersion;
		}

		public override string ToString()
		{
			return string.Format("ExpectedMementoVersion: {0}, ActualMementoVersion: {1}, {2}",
				ExpectedMementoVersion, ActualMementoVersion, base.ToString());
		}
	}
}
